#!/usr/bin/perl
# Client-Anwendung fuer Documation M200-PC-UC-Protokoll
# fuer Linux. Gibt Jones-Cardfile auf STDOUT aus.
# Sven, 18.03.2012

# ubuntu: libdevice-serialport-perl
use Device::SerialPort;
use Data::Dumper;
use strict;

$|++; # Disable output buffer (always a good idea)

my $dev_file = "/dev/ttyS0";
my $dev = new Device::SerialPort($dev_file) or die "Cannot open $dev_file: $!\n";

$dev->baudrate(38400);
$dev->databits(8);
$dev->stopbits(1);
$dev->handshake("none"); # oder "rts"

print STDERR "Got it.\n";

print "H80"; # Jones card prefix
$dev->write("5");

while(1) {
	my $line = read_line($dev);
	next if($line =~ /^\s*$/);
	my ($cmd, $text) = parse_line($line);

	if($cmd >= 100 && $cmd < 200) {
		print STDERR "[INFO $cmd] $text\n";
	} elsif($cmd >= 200 && $cmd < 300) {
		print STDERR "[ACK $cmd] $text\n";
	} elsif($cmd >= 300 && $cmd < 400) {
		print STDERR "[SIGNAL $cmd] $text\n";
	} elsif($cmd >= 400 && $cmd < 500) {
		print STDERR "[DEUBG $cmd] $text\n";
	} elsif($cmd >= 500 && $cmd < 600) {
		die "Client implementation error: Cannot parse Hex output ($line)!\n";
	} elsif($cmd >= 600 && $cmd < 700) {
		die "Client implementation error: Cannot parse Debug output ($line)!\n";
	} elsif($cmd >= 800 && $cmd < 900) {
		# Binary Jones output. I can parse that!
		if($cmd == 800) {
			print STDERR "[JONES] Starting Binary Card\n";

			# print generic card prefix
			print chr(0x80), chr(0x80), chr(0x80);
			my $cardlen = 123; # binary jones format card length
			my $crap_after = 120; # da wird muell vom uc ausgegeben
			my $read = 0;
			while($read < $cardlen) {
				my ($count, $s) = $dev->read(1);
				if($count == 1) {
					print $s if($read < $crap_after);
					$read++;
				}
			}

			# exactly after the 808 must come
			my $line = read_line($dev);
			if($line =~ /^808/) {
				print STDERR "[OK] Card read\n";
			} else {
				print STDERR "[JONES ERROR] Error, missing 808\n";
			}
		} elsif($cmd == 808) {
			print STDERR "Finished Card ACK. Would not expect here\n";
		}
	} elsif($cmd >= 900 && $cmd <= 999) {
		print STDERR "[SERVER CRASH] $text\n";
	} else {
		print STDERR "[PROTO ERROR] Command $cmd not understood ($text)\n";
	}
}

sub read_line {
	my $buffer = '';
	my $c = ''; my $l = ''; # current char, last char
	my $hardlimit = 1000; # max lines
	my $count = 0;
	my $device = shift;
	while(1) {
		$c = $device->read(1);
		$buffer .= $c;
		return($buffer) if($c eq '' and ++$count > $hardlimit);
		return($buffer) if($l eq "\r" or $c eq "\n");
		$l = $c;
	}
}

sub parse_line {
	my $line = shift;
	print STDERR "[PROTO ERROR] Bad line: '$line'\n"
		unless($line =~ /^\s*(\d{3})\s*(.*)\s*$/);
	return( $1, $2 ); # tuple ($number, $text)
}
